grammar Parser;

WS: [ \t\r\n]+ -> channel(HIDDEN) ;
COMMENT: '/*' .*? '*/' -> skip ;
LINE_COMMENT: '//' ~[\r\n]* -> skip ;

fragment DIGIT: [0-9] ;
INT: DIGIT+ ;
FLOAT: INT '.' INT ;
fragment ESC: '\\"' | '\\\\' ;
STRING: '"' ( ESC | ~[\\"] )* '"' ;
BOOL: 'true' | 'false' ;

ID: [a-zA-Z_][a-zA-Z0-9_]* ;

literal: INT | FLOAT | STRING | BOOL ;
ident: ID ;

params: (exprs_+=expr (',' exprs_+=expr)*)? ;
call: name=ident '(' params_=params ')' ;
group:
    ident              # groupDefault
  | call               # groupDefault
  | literal            # groupDefault
  | '(' res_=expr ')'  # groupRule
;
expr:
//    op=('not' | '+' | '-') arg=expr                     #un_op
//  |
    arg1_=expr op_=('*' | '/' | '%') arg2_=expr           # binOpRule
  | arg1_=expr op_=('+' | '-') arg2_=expr                 # binOpRule
  | arg1_=expr op_=('>' | '>=' | '<' | '<=') arg2_=expr   # binOpRule
  | arg1_=expr op_=('==' | '!=') arg2_=expr               # binOpRule
  | arg1_=expr op_='&&' arg2_=expr                        # binOpRule
  | arg1_=expr op_='||' arg2_=expr                        # binOpRule
  | group                                                 # exprDefault
;

type: ident ;
varDecl: ident | assign ;
varsDecl: type_=type vars_+=varDecl (',' vars_+=varDecl)* ;

paramDecl: type_=type name_=ident() ;
funcDecl: type_=type name_=ident() '(' (params_+=paramDecl (',' params_+=paramDecl)*)? ')' body_=stmt ;

assign: name_=ident '=' val_=expr ;
stmt1:
    assign
  | call
  | varsDecl
;
stmt2:
    '{' res_=stmtList '}'                                                   # compoundRule
  | 'if' '(' cond_=expr ')' thenStmt_=stmt ('else' elseStmt_=stmt)?         # ifRule
  | 'while' '(' cond_=expr ')' body_=stmt                                   # whileRule
  | 'for' '(' init_=stmt1? ';' cond_=expr? ';' step_=stmt1? ')' body_=stmt  # forRule
  | funcDecl                                                                # stmt2Default
;
stmt:
    res_=stmt1 ';'  # stmtRule
  | stmt2           # stmtDefault
;
stmtList: (stmts_+=stmt ';'*)* ;
program: res_=stmtList EOF ;
